package eu.eudat.file.transformer.model.rda.mapper;

import eu.eudat.commonmodels.enums.ContactInfoType;
import eu.eudat.commonmodels.models.DmpUserModel;
import eu.eudat.commonmodels.models.UserContactInfoModel;
import eu.eudat.commonmodels.models.reference.ReferenceModel;
import eu.eudat.file.transformer.model.rda.Contributor;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

@Component
public class DmpUserContributorRDAMapper {
	private final ContributorIdRDAMapper contributorIdRDAMapper;

	public DmpUserContributorRDAMapper(ContributorIdRDAMapper contributorIdRDAMapper) {
		this.contributorIdRDAMapper = contributorIdRDAMapper;
	}

	public Contributor toRDA(DmpUserModel userDMP) {
		if (userDMP == null) return null;
		if (userDMP.getUser() == null) throw new IllegalArgumentException("User is missing");
		if (userDMP.getUser().getName() == null) throw new IllegalArgumentException("Contributor Name is missing");
		
		Contributor rda = new Contributor();
		rda.setContributorId(contributorIdRDAMapper.toRDA(userDMP.getUser().getId().toString()));
		
		rda.setName(userDMP.getUser().getName());
		UserContactInfoModel emailContact = userDMP.getUser().getContacts() == null ? null : userDMP.getUser().getContacts().stream().filter(userContactInfo -> userContactInfo.getType().equals(ContactInfoType.Email)).findFirst().orElse(null);
		if (emailContact != null) {
			rda.setMbox(emailContact.getValue());
		}
		rda.setRole(new HashSet<>(List.of(userDMP.getRole().name())));
		return rda;
	}

	public List<Contributor> toRDAs(List<DmpUserModel> userDMPs) {
		if (userDMPs == null) return null;
		List<Contributor> items = new ArrayList<>();
		for (DmpUserModel userDMP : userDMPs){
			Contributor item = this.toRDA(userDMP);
			if (item != null) items.add(item);
		}
		return items;
	}
}

