package eu.eudat.file.transformer.model.rda.mapper;

import eu.eudat.commonmodels.models.reference.ReferenceModel;
import eu.eudat.commonmodels.models.reference.ReferenceTypeModel;
import eu.eudat.file.transformer.model.FundingModel;
import eu.eudat.file.transformer.model.rda.Funding;
import eu.eudat.file.transformer.service.rdafiletransformer.RdaFileTransformerServiceProperties;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;


@Component
public class FundingRDAMapper {

	private final GrantIdRDAMapper grantIdRDAMapper;
	private final FunderIdRDAMapper funderIdRDAMapper;
	private final RdaFileTransformerServiceProperties rdaFileTransformerServiceProperties;

	public FundingRDAMapper(GrantIdRDAMapper grantIdRDAMapper, FunderIdRDAMapper funderIdRDAMapper, RdaFileTransformerServiceProperties rdaFileTransformerServiceProperties) {
		this.grantIdRDAMapper = grantIdRDAMapper;
		this.funderIdRDAMapper = funderIdRDAMapper;
		this.rdaFileTransformerServiceProperties = rdaFileTransformerServiceProperties;
	}


	public Funding toRDA(FundingModel model) {
		if (model == null || (model.getGrant() == null && model.getFunder() == null)) return null;
		Funding rda = new Funding();
		String referencePrefix;
		String shortReference;
		int prefixLength = 0;
		if (model.getFunder() != null && model.getFunder().getReference() != null) {
			referencePrefix = model.getFunder().getReference().split(":")[0];
			prefixLength = referencePrefix.length() == model.getFunder().getReference().length() ? referencePrefix.length() - 1 : referencePrefix.length();
			shortReference = model.getFunder().getReference().substring(prefixLength + 1);
			rda.setFunderId(funderIdRDAMapper.toRDA(shortReference));
		} else if (model.getFunder() != null){
			rda.setFunderId(funderIdRDAMapper.toRDA(model.getFunder().getId()));
		}
		if (model.getGrant() != null && model.getGrant() != null) {
			referencePrefix = model.getGrant().getReference().split(":")[0];
			prefixLength = referencePrefix.length() == model.getGrant().getReference().length() ? referencePrefix.length() - 1 : referencePrefix.length();
			shortReference = model.getGrant().getReference().substring(prefixLength + 1);
			rda.setGrantId(grantIdRDAMapper.toRDA(shortReference));
		} else if (model.getGrant() != null) {
			rda.setGrantId(grantIdRDAMapper.toRDA(model.getGrant().getId().toString()));
		}
		return rda;
	}

	public FundingModel toEntity(Funding rda) {
		if (rda == null) return null;
		FundingModel references = new FundingModel();
		ReferenceModel funder = new ReferenceModel();
		ReferenceTypeModel funderReferenceTypeModel = new ReferenceTypeModel();
		funderReferenceTypeModel.setCode(rdaFileTransformerServiceProperties.getFunderReferenceCode());
		funder.setType(funderReferenceTypeModel);
		funder.setReference(rda.getFunderId().getIdentifier());
		references.setFunder(funder);
		ReferenceModel grant = new ReferenceModel();
		ReferenceTypeModel grantReferenceTypeModel = new ReferenceTypeModel();
		grantReferenceTypeModel.setCode(rdaFileTransformerServiceProperties.getGrantReferenceCode());
		grant.setType(grantReferenceTypeModel);
		grant.setReference(rda.getGrantId().getIdentifier());
		references.setGrant(grant);
		return references;
	}

	public List<FundingModel> toEntities(List<Funding> rdas) {
		if (rdas == null) return null;
		List<FundingModel> items = new ArrayList<>();
		for (Funding rda : rdas){
			FundingModel item = this.toEntity(rda);
			if (item != null) items.add(item);
		}
		return items;
	}
}

