package eu.eudat.file.transformer.model.rda.mapper;

import com.fasterxml.jackson.databind.ObjectMapper;
import eu.eudat.file.transformer.model.FundingModel;
import eu.eudat.file.transformer.model.rda.Funding;
import eu.eudat.file.transformer.model.rda.Language;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.*;

@Component
public class LanguageRDAMapper {
	private final static Map<String, String> langMap = new HashMap<>();
	private static final Logger logger = LoggerFactory.getLogger(LanguageRDAMapper.class);

	static {
		try {
			ObjectMapper mapper = new ObjectMapper(); //TODO
			InputStreamReader isr = new InputStreamReader(LanguageRDAMapper.class.getClassLoader().getResource("internal/rda-lang-map.json").openStream(), StandardCharsets.UTF_8);
			langMap.putAll(mapper.readValue(isr, LinkedHashMap.class));
			isr.close();

		} catch (IOException e) {
			logger.error(e.getMessage(), e);
		}
	}

	public Language toRDA(String code) {
		if (code == null || code.isBlank()) return  null;
		return langMap.entrySet().stream().map(entry -> {
			if (entry.getValue().equals(code)) {
				return Language.fromValue(entry.getKey());
			} else {
				return null;
			}
		}).filter(Objects::nonNull).findFirst().orElse(null);
	}

	public String toEntity(Language lang) {
		if (lang == null) return null;
		return langMap.get(lang.value());
	}
	
}
